# Copyright (C) 1999, 2000 Jay Beale
# Copyright (C) 2001, 2002 Hewlett Packard
# Licensed under the GNU General Public License

package Bastille::AccountSecurity;
use lib "/usr/lib";


use Bastille::API;

use Bastille::TestAPI;

# 5/3/2000 -- new version created from old by JJB

#  TO DO:
#        1) Document by copying text from questions file...

#######################################################################
##                     Account Creation/Security                     ##
#######################################################################

&ProtectRhosts;
&PasswordAging;
&RestrictCron;
&SetUmask;
&RootTTYLogins;
&Securetty;
&HPDefaultSecurity;
&ShadowedPasswords;

sub ProtectRhosts {

    if ( &getGlobalConfig("AccountSecurity","protectrhost") eq "Y" ) {

	&ActionLog("# sub ProtectRhosts\n");

	# Disallow rlogin,rsh,rexec via Pluggable Authentication Modules
	    foreach $service ( "rexec","rlogin","rsh" ) {
		if ( -e &getGlobal('DIR', "pamd") . "/$service" ) {
		    &B_prepend_line(&getGlobal('DIR', "pamd") . "/$service","pam_deny","auth      required   pam_deny.so\n");
		}
	    }

	# Deactivate the daemons by removing execute status
	&B_chmod_if_exists(0000,&getGlobal('BIN',"rlogind"));
	&B_chmod_if_exists(0000,&getGlobal('BIN',"rshd"));
	&B_chmod_if_exists(0000,&getGlobal('BIN',"rexecd"));
	
	# Comment out rsh/rlogin/rexec lines from inetd.conf
        # Note: SecureInetd.pm does this too.
	if ( -e &getGlobal('FILE', "inetd.conf") ) {
	    &B_hash_comment_line( &getGlobal('FILE', "inetd.conf"),"rlogind");
	    &B_hash_comment_line( &getGlobal('FILE', "inetd.conf"),"rexecd");
	    &B_hash_comment_line( &getGlobal('FILE', "inetd.conf"),"rshd");
	}   
    }
}


sub PasswordAging {

# Set default password aging, such that accounts are disabled if the
# password is not changed every 180 days.  We use this hopefully to keep
# passwords fresh and automatically disable accounts that haven't been 
# used in a while.  We could create a cron job that parses lastlog looking
# for unused accounts, but that would fail if your lastlog got deleted by
# an attacker or cycled by your log cycler, as the cron job would disable
# many accounts...

    if (&getGlobalConfig("AccountSecurity","passwdage") eq "Y") {
	&ActionLog("# sub Password Aging\n");
	&ActionLog("adding PASS_MAX_DAYS setting to /etc/login.defs\n");
	
        my $got_replaced=&B_replace_line("/etc/login.defs",'^\s*PASS_MAX_DAYS',"PASS_MAX_DAYS   180\n");
        unless ($got_replaced) {
            B_append_line("/etc/login.defs",'^\s*PASS_MAX_DAYS',"PASS_MAX_DAYS   180\n");
        }
    }				      
}



sub RestrictCron {

   if ( &getGlobalConfig("AccountSecurity","cronuser") eq "Y" ) {

       &ActionLog("# sub RestrictCron\n");

	# We do this even if cron isn't installed/enabled, because
	# it doesn't hurt

       # Create an /etc/cron.allow file, thus restricting crontab's use to
       # root (and other users that might be added to this list)

       # Create the cron.allow file, unless it already exists.
       &B_create_file ("/etc/cron.allow");

       # Append a list of accounts that can use cron -- only adding lines
       # if they're not yet in there...

       &B_append_line("/etc/cron.allow",'^root$',"root\n");
       
   }
}

sub SetUmask {
    if ( &getGlobalConfig("AccountSecurity","umaskyn") eq "Y" ) {
	&ActionLog("# sub SetUmask\n");

	my $umask = &getGlobalConfig("AccountSecurity","umask");
	if ( -e &getGlobal('FILE', "profile") ) {
	    &B_append_line( &getGlobal('FILE', "profile"),'^([^#]*)*umask',"umask $umask\n");
	    &B_replace_line( &getGlobal('FILE', "profile"),'^([^#]*)*umask',"umask $umask\n");
	}

	if ( -e &getGlobal('FILE', "rootprofile") ) {
	    &B_append_line( &getGlobal('FILE', "rootprofile"),'^([^#]*)umask',"umask $umask\n");
	    &B_replace_line( &getGlobal('FILE', "rootprofile"),'^([^#]*)umask',"umask $umask\n");
	}

	if ( -e &getGlobal('FILE', "zprofile") ) {
	    &B_append_line(&getGlobal('FILE', "zprofile"),'^([^#]*)umask',"umask $umask\n");
	    &B_replace_line(&getGlobal('FILE', "zprofile"),'^([^#]*)umask',"umask $umask\n");
	}

	if ( -e &getGlobal('FILE', "csh.login") ) {
	     &B_append_line(&getGlobal('FILE', "csh.login"),'^([^#]*)umask',"umask $umask\n");
	     &B_replace_line(&getGlobal('FILE', "csh.login"),'^([^#]*)umask',"umask $umask\n");
	 }

	if (&GetDistro =~ "^HP-UX") {
	    # The system umask on a Linux system will vary, but isn't in /etc/default/security.
	    my $system_umask = &getGlobalConfig("AccountSecurity","system_umask");

	    if (defined $system_umask) {
		my $secfile = &getGlobal('FILE','security');
		&B_set_value('UMASK', $system_umask, $secfile);
	    }
	}
    }
}

sub RootTTYLogins {
    if ( &getGlobalConfig("AccountSecurity","rootttylogins") eq "Y" ) {
	&ActionLog("# sub RootTTYLogins\n");
	
	my $tty;
	foreach $tty (1,2,3,4,5,6,7,8,9,0) {
	    &B_delete_line("/etc/securetty","tty$tty");
	}

	# stop root from logging in via xdm as well
	#

	#&B_append_line("/etc/X11/xdm/Xresources","xlogin\.Login\.allowRootLogin","xlogin.Login.allowRootLogin: false");
	&B_create_file("/etc/bastille-no-login");
	&B_append_line("/etc/bastille-no-login",'\broot\b',"root\n");
	&B_prepend_line("/etc/pam.d/xdm",'bastille-no-login',"auth\trequired\t/lib/security/pam_listfile.so onerr=succeed item=user sense=deny file=/etc/bastille-no-login");

	# stop root from logging in via gdm
	#&B_replace_line("/etc/X11/gdm/gdm.conf",'^\s*AllowRoot\b',"AllowRoot=0\n");
	&B_prepend_line("/etc/pam.d/gdm",'bastille-no-login',"auth\trequired\t/lib/security/pam_listfile.so onerr=succeed item=user sense=deny file=/etc/bastille-no-login");
	# stop root from logging in via kdm
	&B_prepend_line("/etc/pam.d/kde",'bastille-no-login',"auth\trequired\t/lib/security/pam_listfile.so onerr=succeed item=user sense=deny file=/etc/bastille-no-login");
    

#    } #Shouldn't make changes when the user picks "no" on a question.  Also, this changes 
#      #systems even when the question wasn't asked (other distros) "else" commented out Jul 23 2001 - RwF
#    else {
#	my $tty;
#        foreach $tty (1,2,3,4,5,6) {
#            &B_append_line("/etc/securetty","tty$tty","tty$tty\n");
 #       }
    }
    #BUG  we need to allow the user to reverse this on gdm,xdm and kdm too
	 	
}

sub RestrictUserView {
    
    # This routine restricts the kdm/gdm userview in Linux-Mandrake
    # Motivation: Compabilitity with msec

    if (&getGlobalConfig("AccountSecurity","forbiduserview") eq "Y") {
	&ActionLog("# sub RestrictUserView\n");

	&B_replace_line("/usr/share/config/kdmrc",'^UserView=',"UserView=false=n");
	&B_replace_line("/etc/X11/gdm/gdm.conf","^Browser=","Browser=0\n");
    }
}

sub Securetty { 

    if (&getGlobalConfig("AccountSecurity","create_securetty") eq "Y") {
	
	&ActionLog("# sub Securetty\n");
	# echo console > /etc/securetty
        # chmod 400 /etc/securetty
	
	unless ( -e &getGlobal('FILE', "securetty") ) {
	    &B_create_file(&getGlobal('FILE', "securetty"));
	}
	&B_blank_file(&getGlobal('FILE', "securetty"),'a$b');
	&B_append_line(&getGlobal('FILE', "securetty"),"console","console\n");

    }
    
}


sub HPDefaultSecurity {

    if (&GetDistro =~ "^HP-UX") {
	my $secfile = &getGlobal('FILE','security');

	# options to disallow logins by normal users
	# ABORT_LOGIN_ON_MISSING_HOMEDIR - don't login if homedir is missing
	# NOLOGIN - don't login if /etc/nologin exists
	# BOOT_AUTH - require root password to enter single user mode.
	foreach my $param ("ABORT_LOGIN_ON_MISSING_HOMEDIR", "NOLOGIN", "BOOT_AUTH") {
	    if (&getGlobalConfig("AccountSecurity","$param") eq "Y") {
		&B_set_value("$param", '1', $secfile);
	    }
	}
	
	### Set password policies
	if (&getGlobalConfig("AccountSecurity","passwordpolicies") eq "Y") {
	    foreach my $param ("MIN_PASSWORD_LENGTH",
			       "PASSWORD_HISTORY_DEPTH",
			       "PASSWORD_MAXDAYS",
			       "PASSWORD_MINDAYS",
			       "PASSWORD_WARNDAYS",) {
		&B_set_value("$param", 
			     &getGlobalConfig("AccountSecurity", $param), $secfile);
	    }
	}
	
	if (&getGlobalConfig("AccountSecurity", "NUMBER_OF_LOGINS_ALLOWEDyn") eq "Y") {
	    &B_set_value("NUMBER_OF_LOGINS_ALLOWED", 
			 &getGlobalConfig("AccountSecurity", "NUMBER_OF_LOGINS_ALLOWED"), $secfile);
	}
	
	if (&getGlobalConfig("AccountSecurity", "SU_DEFAULT_PATHyn") eq "Y") {
	    &B_set_value("SU_DEFAULT_PATH", 
			 &getGlobalConfig("AccountSecurity", "SU_DEFAULT_PATH"), $secfile);
	}
    }	
#   The SU_ROOT_GROUP option is too dangerous unless we really do it
#   right.  Even if we add the users, they wouldn't have passwords, 
#   so you couldn't login to those accounts.  And, you can't login
#   directly as root because of the securetty setting.
#
#   What follows here is what I had done before realizing this.
#
#   Here is an excerpt which should be placed after SU_DEFAULT_PATH
#   in Questions.txt with the appropriate links (without comments)
#
#SU_ROOT_GROUP=wheel
#LABEL: SU_ROOT_GROUPyn
#QUESTION: "Do you want to restrict accounts which can become superuser?"
#SHORT_EXP: "If you don't do this, all non superusers are allowed to 
#su to root without being bound by root group restrictions.  (They still
#must know the root password to become root.)  Be careful here, because
#if you disallowed root logins already, you could get locked out of your
#box if you can't login to one of these accounts."
#DEFAULT_ANSWER: "N"
#REQUIRE_DISTRO: HP-UX11.22 HP-UX11.23
#YN_TOGGLE: 1
#REG_EXP: "^[YN]$"
#REQUIRE_DISTRO: HP-UX11.22 HP-UX11.23
#YES_CHILD: SU_ROOT_GROUP
#NO_CHILD: SU_DEFAULT_PATHyn
#SKIP_CHILD: rootttylogins
#PROPER_PARENT: NOLOGIN
#
#LABEL: SU_ROOT_GROUP
#QUESTION: "Enter the name of the \"su root group\""
#SHORT_EXP: "The SU_ROOT_GROUP parameter defines the root group name for the su
#command.  Refer to su(1).
#
#SU_ROOT_GROUP=group_name The root group name is set to
#the specified symbolic group name.  The su command
#enforces the restriction that a non-superuser must be a
#member of the specified root group in order to be
#allowed to su to root.  This does not alter password
#checking.
#
#This is similar to a feature of some BSD-style Unices which require that
#a user is part of a trusted group to use the \"su\" command."
#DEFAULT_ANSWER: "wheel"
#YN_TOGGLE: 0
#REG_EXP: "^[a-zA-Z]+$"
#EXPL_ANS: "wheel"
#REQUIRE_DISTRO: HP-UX11.22 HP-UX11.23
#YES_CHILD: su_root_group_users
#SKIP_CHILD: rootttylogins
#PROPER_PARENT: SU_ROOT_GROUPyn
#
#LABEL: su_root_group_users
#QUESTION: "Which users do you want in the \"su root group\"?"
#SHORT_EXP: "It is important to define users that you trust to become
#root.  Otherwise, you could be locked out of your system.  These
#should all be valid users on your system.  You can list multiple
#usernames by separating them with commas."
#YN_TOGGLE: 0
#REG_EXP: "^[a-zA-Z]+(,[a-zA-Z])*$"
#EXPL_ANS: "joe,sue,fred"
#REQUIRE_DISTRO: HP-UX11.22 HP-UX11.23
#YES_CHILD: SU_DEFAULT_PATHyn
#SKIP_CHILD: rootttylogins
#PROPER_PARENT: SU_ROOT_GROUPyn


#    Here is the code outline for this functionality:
#
#    if (&getGlobalConfig("AccountSecurity",
#                         "SU_ROOT_GROUPyn") eq "Y") {
#        my $userlist=&getGlobalConfig("AccountSecurity", "su_root_group_users");
#        my @users = split($userlist, ',');
#
#        for my $user (@users) {
#           # see if the user exists.  If not, add them, and make sure
#           # the password gets set before root logs out.
#
#        }
#
#        my $su_root_group=&getGlobalConfig("AccountSecurity", 
#                                      "SU_ROOT_GROUP"), $secfile);
#
#        # unless the group exists, add it
#        &B_System("$groupadd $su_root_group", "$groupdel $su_root_group");
#
#        &B_set_value("$param", $su_root_group);
#    }


}


sub ShadowedPasswords {
    
    if( &getGlobalConfig('AccountSecurity',"shadowed_passwords") eq "Y") {
	# configuration files in which shadowed passwords depend
        my $nsswitch_conf = &getGlobal('FILE',"nsswitch.conf");
	# binaries use to convert to a shadowed password
	my $pwconv = &getGlobal('BIN',"pwconv");
	my $echo = &getGlobal('BIN','echo'); # the echo is used to pipe a yes into the pwconv program as
	                                  # pwconv requires user interaction.
	# the binary used in a system revert.
	my $pwunconv = &getGlobal('BIN',"pwunconv");
	#check the password file for nis usage and if the nis client
	#or server is running.  
	if(-e $nsswitch_conf) {
	    # check the file for nis, nis+, ldap, compat, or dce usage.
	    if(&B_match_line($nsswitch_conf, '^\s*passwd:.+(nis|nisplus|ldap|dce|compat)')) {
		my $shadowTODO = "\n---------------------------------\nShadowing Password File:\n" . 
		                 "---------------------------------\n" .
		                 "This version of password shadowing does not support any repository other\n" .
		                 "than files. In order to convert your password database to shadowed passwords\n" .
				 "there can be no mention of nis, nisplus, ldap, compat, or dce in the passwd\n" .
				 "field of the \"$nsswitch_conf\" file.  Please make the nessecary edits to\n" . 
				 "the $nsswitch_conf file and run Bastille again using the command:\n" .
				 "\"bastille -b\"\n";
		# Adding the shadowTODO comment to the TODO list.
		&B_TODO("$shadowTODO");
		# Notifing the user that the shadowed password coversion has failed.
		&ErrorLog("Password Shadowing Conversion Failed\n" .
			  "$shadowTODO");
		# exiting the subroutine.
		return 0;
	    }
	    
	}

	# convert the password file to a shadowed repository.
	if( &B_System("$echo \"yes\" | $pwconv","$pwunconv") ) {
	    &B_TODO( "\n---------------------------------\nShadowing Password File:\n" . 
		     "---------------------------------\n" .
		     "Your password file has been converted to use password shadowing.\n" .
		     "This version of password shadowing does not support any repository other\n" .
		     "than files. There can be no mention of nis, nisplus, ldap, compat, or dce\n" .
		     "in the passwd field of the \"$nsswitch_conf\" file.\n\n" );
	}


    }
}







1;

